local E, C, L, DB = unpack(select(2, ...)) -- Import Functions/Constants, Config, Locales
if C["skin"].enable ~= true or C["skin"].friends ~= true then return end

--Tab Regions
local tabs = {
	"LeftDisabled",
	"MiddleDisabled",
	"RightDisabled",
	"Left",
	"Middle",
	"Right",
}

--Social Frame
local function SkinSocialHeaderTab(tab)
	if not tab then return end
	for _, object in pairs(tabs) do
		local tex = _G[tab:GetName()..object]
		tex:SetTexture(nil)
	end
	tab:GetHighlightTexture():SetTexture(nil)
	tab.backdrop = CreateFrame("Frame", nil, tab)
	tab.backdrop:SetTemplate("Default")
	tab.backdrop:SetFrameLevel(tab:GetFrameLevel() - 1)
	tab.backdrop:Point("TOPLEFT", 3, -8)
	tab.backdrop:Point("BOTTOMRIGHT", -6, 0)
end

local function LoadSkin()
	E.SkinScrollBar(FriendsFrameFriendsScrollFrameScrollBar, 5)
	E.SkinScrollBar(WhoListScrollFrameScrollBar, 5)
	E.SkinScrollBar(ChannelRosterScrollFrameScrollBar, 5)
	local StripAllTextures = {
		"ScrollOfResurrectionSelectionFrame",
		"ScrollOfResurrectionSelectionFrameList",
		"FriendsListFrame",
		"FriendsTabHeader",
		"FriendsFrameFriendsScrollFrame",
		"WhoFrameColumnHeader1",
		"WhoFrameColumnHeader2",
		"WhoFrameColumnHeader3",
		"WhoFrameColumnHeader4",
		"ChannelListScrollFrame",
		"ChannelRoster",
		"FriendsFramePendingButton1",
		"FriendsFramePendingButton2",
		"FriendsFramePendingButton3",
		"FriendsFramePendingButton4",
		"ChannelFrameDaughterFrame",
		"AddFriendFrame",
		"AddFriendNoteFrame",
	}

	local KillTextures = {
		"FriendsFrameBroadcastInputLeft",
		"FriendsFrameBroadcastInputRight",
		"FriendsFrameBroadcastInputMiddle",
		"ChannelFrameDaughterFrameChannelNameLeft",
		"ChannelFrameDaughterFrameChannelNameRight",
		"ChannelFrameDaughterFrameChannelNameMiddle",
		"ChannelFrameDaughterFrameChannelPasswordLeft",
		"ChannelFrameDaughterFrameChannelPasswordRight",
		"ChannelFrameDaughterFrameChannelPasswordMiddle",
	}

	FriendsFrameInset:StripTextures()
	WhoFrameListInset:StripTextures()
	WhoFrameEditBoxInset:StripTextures()
	ChannelFrameRightInset:StripTextures()
	ChannelFrameLeftInset:StripTextures()
	LFRQueueFrameListInset:StripTextures()
	LFRQueueFrameRoleInset:StripTextures()
	LFRQueueFrameCommentInset:StripTextures()

	local buttons = {
		"FriendsFrameAddFriendButton",
		"FriendsFrameSendMessageButton",
		"WhoFrameWhoButton",
		"WhoFrameAddFriendButton",
		"WhoFrameGroupInviteButton",
		"ChannelFrameNewButton",
		"FriendsFrameIgnorePlayerButton",
		"FriendsFrameUnsquelchButton",
		"FriendsFramePendingButton1AcceptButton",
		"FriendsFramePendingButton1DeclineButton",
		"FriendsFramePendingButton2AcceptButton",
		"FriendsFramePendingButton2DeclineButton",
		"FriendsFramePendingButton3AcceptButton",
		"FriendsFramePendingButton3DeclineButton",
		"FriendsFramePendingButton4AcceptButton",
		"FriendsFramePendingButton4DeclineButton",
		"ChannelFrameDaughterFrameOkayButton",
		"ChannelFrameDaughterFrameCancelButton",
		"AddFriendEntryFrameAcceptButton",
		"AddFriendEntryFrameCancelButton",
		"AddFriendInfoFrameContinueButton",
		"ScrollOfResurrectionSelectionFrameAcceptButton",
		"ScrollOfResurrectionSelectionFrameCancelButton",
	}

	for _, button in pairs(buttons) do
		E.SkinButton(_G[button])
	end

	for _, texture in pairs(KillTextures) do
		_G[texture]:Kill()
	end

	for _, object in pairs(StripAllTextures) do
		_G[object]:StripTextures()
	end

	for i=1, FriendsFrame:GetNumRegions() do
		local region = select(i, FriendsFrame:GetRegions())
		if region:GetObjectType() == "Texture" then
			region:SetTexture(nil)
			region:SetAlpha(0)
		end
	end

	E.SkinEditBox(AddFriendNameEditBox)
	AddFriendFrame:SetTemplate("Transparent")
	ScrollOfResurrectionSelectionFrame:SetTemplate("Transparent")
	ScrollOfResurrectionSelectionFrameList:SetTemplate("Default")
	E.SkinScrollBar(ScrollOfResurrectionSelectionFrameListScrollFrameScrollBar, 4)
	E.SkinEditBox(ScrollOfResurrectionSelectionFrameTargetEditBox)

	--Who Frame
	local function UpdateWhoSkins()
		WhoListScrollFrame:StripTextures()
	end
	--Channel Frame
	local function UpdateChannel()
		ChannelRosterScrollFrame:StripTextures()
	end
	--BNet Frame
	FriendsFrameBroadcastInput:CreateBackdrop("Default")
	ChannelFrameDaughterFrameChannelName:CreateBackdrop("Default")
	ChannelFrameDaughterFrameChannelPassword:CreateBackdrop("Default")

	ChannelFrame:HookScript("OnShow", UpdateChannel)
	hooksecurefunc("FriendsFrame_OnEvent", UpdateChannel)

	WhoFrame:HookScript("OnShow", UpdateWhoSkins)
	hooksecurefunc("FriendsFrame_OnEvent", UpdateWhoSkins)

	ChannelFrameDaughterFrame:CreateBackdrop("Transparent")

	FriendsFrame:SetTemplate('Transparent')

	E.SkinCloseButton(ChannelFrameDaughterFrameDetailCloseButton,ChannelFrameDaughterFrame)
	E.SkinCloseButton(FriendsFrameCloseButton,FriendsFrame.backdrop)
	E.SkinDropDownBox(WhoFrameDropDown,150)
	E.SkinDropDownBox(FriendsFrameStatusDropDown,70)

	--Bottom Tabs
	for i=1, 4 do
		E.SkinTab(_G["FriendsFrameTab"..i])
	end

	for i=1, 3 do
		SkinSocialHeaderTab(_G["FriendsTabHeaderTab"..i])
	end

	local function Channel()
		for i=1, MAX_DISPLAY_CHANNEL_BUTTONS do
			local button = _G["ChannelButton"..i]
			if button then
				button:StripTextures()
				button:SetHighlightTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight")
				
				_G["ChannelButton"..i.."Text"]:SetFont(C["media"].font, 12)
			end
		end
	end
	hooksecurefunc("ChannelList_Update", Channel)

	--View Friends BN Frame
	FriendsFriendsFrame:CreateBackdrop("Transparent")

	local StripAllTextures = {
		"FriendsFriendsFrame",
		"FriendsFriendsList",
		"FriendsFriendsNoteFrame",
	}

	local buttons = {
		"FriendsFriendsSendRequestButton",
		"FriendsFriendsCloseButton",
	}

	for _, object in pairs(StripAllTextures) do
		_G[object]:StripTextures()
	end

	for _, button in pairs(buttons) do
		E.SkinButton(_G[button])
	end

	E.SkinEditBox(FriendsFriendsList)
	E.SkinEditBox(FriendsFriendsNoteFrame)
	E.SkinDropDownBox(FriendsFriendsFrameDropDown,150)

	BNConversationInviteDialog:StripTextures()
	BNConversationInviteDialog:CreateBackdrop("Transparent")
	BNConversationInviteDialogList:StripTextures()
	BNConversationInviteDialogList:SetTemplate("Default")
	E.SkinButton(BNConversationInviteDialogInviteButton)
	E.SkinButton(BNConversationInviteDialogCancelButton)

	for i=1, BN_CONVERSATION_INVITE_NUM_DISPLAYED do
	E.SkinCheckBox(_G["BNConversationInviteDialogListFriend"..i].checkButton)
	end

	FriendsTabHeaderSoRButton:SetTemplate('Default')
	FriendsTabHeaderSoRButton:StyleButton()
	FriendsTabHeaderSoRButtonIcon:SetDrawLayer('OVERLAY')
	FriendsTabHeaderSoRButtonIcon:SetTexCoord(.08, .92, .08, .92)
	FriendsTabHeaderSoRButtonIcon:ClearAllPoints()
	FriendsTabHeaderSoRButtonIcon:Point('TOPLEFT', 2, -2)
	FriendsTabHeaderSoRButtonIcon:Point('BOTTOMRIGHT', -2, 2)
	FriendsTabHeaderSoRButton:Point('TOPRIGHT', FriendsTabHeader, 'TOPRIGHT', -8, -56)

	E.SkinScrollBar(FriendsFrameIgnoreScrollFrameScrollBar, 4)
	E.SkinScrollBar(FriendsFramePendingScrollFrameScrollBar, 4)

	IgnoreListFrame:StripTextures()
	PendingListFrame:StripTextures()

	ScrollOfResurrectionFrame:StripTextures()
	E.SkinButton(ScrollOfResurrectionFrameAcceptButton)
	E.SkinButton(ScrollOfResurrectionFrameCancelButton)

	ScrollOfResurrectionFrameTargetEditBoxLeft:SetTexture(nil)
	ScrollOfResurrectionFrameTargetEditBoxMiddle:SetTexture(nil)
	ScrollOfResurrectionFrameTargetEditBoxRight:SetTexture(nil)
	ScrollOfResurrectionFrameNoteFrame:StripTextures()
	ScrollOfResurrectionFrameNoteFrame:SetTemplate("Transparent")
	ScrollOfResurrectionFrameTargetEditBox:SetTemplate("Default")
	ScrollOfResurrectionFrame:SetTemplate("Transparent")

	for i=1, BN_CONVERSATION_INVITE_NUM_DISPLAYED do
		E.SkinCheckBox(_G["BNConversationInviteDialogListFriend"..i].checkButton)
	end

	--Report Name Frame
	ReportPlayerNameDialog:StripTextures()
	ReportPlayerNameDialog:CreateBackdrop("Transparent")
	E.SkinButton(ReportPlayerNameDialogReportButton)
	E.SkinButton(ReportPlayerNameDialogCancelButton)

	--Report Cheating Frame
	ReportCheatingDialog:StripTextures()
	ReportCheatingDialog:CreateBackdrop("Transparent")
	E.SkinButton(ReportCheatingDialogReportButton)
	E.SkinButton(ReportCheatingDialogCancelButton)

	-- RealID Cross Realm Invite Button
	for i=1, 100 do
	    local CrossRealm = _G["FriendsFrameFriendsScrollFrameButton"..i.."TravelPassButton"]
		if CrossRealm then
			local button = CrossRealm:GetNormalTexture():GetTexture()

			CrossRealm:StripTextures()
			CrossRealm:SetNormalTexture(button)

			CrossRealm:CreateBackdrop("Transparent")
			CrossRealm:Size(20, 29)
			CrossRealm.backdrop:SetAllPoints()

			local point, relatedTo, point2, x, y = CrossRealm:GetPoint()
			CrossRealm:Point(point, relatedTo, point2, 1, y)
		end
	end
end

tinsert(E.SkinFuncs["ElvUI"], LoadSkin)