local quests = {
  [71635] = 29443,
  [71636] = 29444,
  [71637] = 29445,
  [71638] = 29446,
  [71715] = 29451,
  [71716] = 29464,
  [71951] = 29456,
  [71952] = 29457,
  [71953] = 29458,
}

local addonName, addon = ..., CreateFrame("Frame")
local completed, db, pdb, realm, faction, player, item, temp = {}, {}, {}
DarkmoonFaireQuestsDB = {}

local function queryCompletedQuests(addon, event)
  if event == "QUEST_QUERY_COMPLETE" then
    GetQuestsCompleted(completed)
    for k, v in pairs(quests) do
      pdb[k] = completed[v]
    end
  end
end

local function appendTooltipNames(tip)
  temp = ""
  for k, v in pairs(db[realm][faction]) do
    if k ~= player and (not v[item]) then
      temp = temp..k..", "
    end
  end
  if temp:len() > 0 then
    tip:AddLine("Needed by "..temp:sub(0, temp:len()-2)..".", 0, 1, 1, 1)
  end
end

local function updateTooltip(tip)
  if type(pdb) ~= "table" then return end
  if type(tip.GetItem) ~= "function" or (not tip:GetItem()) then return end
  item = tonumber(select(2, tip:GetItem()):match("item:(%d+)"))
  if item and quests[item] then
    QueryQuestsCompleted()
    if not next(completed) then
      tip:AddLine("Loading completed quests, please wait...", 1, 1, 0, 0)
    else
      if pdb[item] then
        tip:AddLine("You have completed this quest!", 1, 0, 0, 0)
      else
        tip:AddLine("You have not yet completed this quest.", 0, 1, 0, 0)
      end
    end
    appendTooltipNames(tip)
  end
end

addon:SetScript("OnEvent", function(addon, event, name)
  if event == "ADDON_LOADED" and name == addonName then
    addon:UnregisterEvent("ADDON_LOADED")
    realm, faction, player = GetRealmName(), select(2, UnitFactionGroup("player")), UnitName("player")
    DarkmoonFaireQuestsDB = type(DarkmoonFaireQuestsDB) == "table" and DarkmoonFaireQuestsDB or {}
    db = DarkmoonFaireQuestsDB
    db[realm] = db[realm] or {}
    db[realm][faction] = db[realm][faction] or {}
    db[realm][faction][player] = db[realm][faction][player] or {}
    pdb = db[realm][faction][player]
    addon:HookScript("OnEvent", queryCompletedQuests)
    addon:RegisterEvent("QUEST_QUERY_COMPLETE")
    GameTooltip:HookScript("OnTooltipSetItem", updateTooltip)
    ItemRefTooltip:HookScript("OnTooltipSetItem", updateTooltip)
    QueryQuestsCompleted()
  end
end)

addon:RegisterEvent("ADDON_LOADED")
