local E, C, L, DB = unpack(ElvUI)

--[[
ElvUI FactionChampioning, by Nevian @ Xavius-EU
]]

local addon, ns=...
config = ns.config

local btnw = config.buttonWidth
local btnh = config.buttonHeight
local btns = config.buttonSpacing
local font = C["media"].font
local faction = select(1, UnitFactionGroup("player"))

-- set reaction colors
E.ReactionColor = {
	[1] = { 222/255, 95/255,  95/255 }, 
	[2] = { 222/255, 95/255,  95/255 }, 
	[3] = { 222/255, 95/255,  95/255 }, 
	[4] = { 218/255, 197/255, 92/255 },
	[5] = { 75/255,  175/255, 76/255 },
	[6] = { 75/255,  175/255, 101/255 },
	[7] = { 75/255,  175/255, 126/255 },
	[8] = { 75/255,  175/255, 151/255 },
}

E.FactionStandings = {
	[1] = "Hated", 
	[2] = "Hostile", 
	[3] = "Unfriendly", 
	[4] = "Neutral",
	[5] = "Friendly",
	[6] = "Honored",
	[7] = "Revered",
	[8] = "Exalted",
}

-- [index] = { factionID, tabardID },
E.HordeFactions = {
	[1] = { 76, 45581 }, -- Orgrimmar
	[2] = { 68, 45583 }, -- Undercity
	[3] = { 81, 45584 }, -- Thunder Bluff
	[4] = { 530, 45582 }, -- Darkspear Trolls
	[5] = { 911, 45585 }, -- Silvermoon City
	[6] = { 1133, 64884 }, -- Bilgewater Cartel
}

E.AllianceFactions = {
	[1] = { 72, 45574 }, -- Stormwind
	[2] = { 47, 45577 }, -- Ironforge
	[3] = { 69, 45579 }, -- Darnassus
	[4] = { 54, 45578 }, -- Gnomeregan
	[5] = { 930, 45580 }, -- Exodar
	[6] = { 1134, 64882 }, -- Gilneas
}

E.WrathFactions = {
	[1] = { 1090, 43157 }, -- Kirin Tor
	[2] = { 1098, 43155 }, -- Knights of the Ebon Blade
	[3] = { 1091, 43156 }, -- The Wyrmrest Accord
	[4] = { 1106, 43154 }, -- Argent Crusade
}

E.CataFactions = {
	[1] = { 1171, 65907 }, -- Therazane
	[2] = { 1158, 65906 }, -- Guardians of Hyjal
	[3] = { 1135, 65905 }, -- The Earthen Ring
	[4] = { 1173, 65904 }, -- Ramkahen
	[5] = { 1172, 65908 }, -- Dragonmaw Clan
	[6] = { 1168, 69210 }, -- Guild
	[7] = { 1174, 65909 }, -- Wildhammer Clan
}

-- faction information (name, current rep, etc.)
E.FactionInfo = function(self)
	local factionName, _, factionStanding, barMin, barMax, barValue, _, _, _, _, _, _, _ = GetFactionInfoByID(self.factionID)

	local standingName = E.FactionStandings[factionStanding]

	self.text:SetText(""..factionName..": "..standingName.."")

	if factionStanding == 8 and math.abs(barValue-barMin) == 999 then self.text:SetText(""..factionName..": Done") end

	self.bar:SetMinMaxValues(0, math.abs(barMax-barMin))
	self.bar:SetValue(math.abs(barValue-barMin))

	local c = E.ReactionColor[factionStanding]
	local r, g, b = c[1], c[2], c[3]

	self.bar:SetStatusBarColor(r, g, b)
	self:SetScript("OnEnter", function() self.bar:SetStatusBarColor(r*1.25, g*1.25, b*1.25) end)
	self:SetScript("OnLeave", function() self.bar:SetStatusBarColor(r, g, b) end)
end

-- script to switch watched faction and tabard
E.FactionSwitch = function(self)
	local factionName = select(1, GetFactionInfoByID(self.factionID))
	local factionStanding = select(3, GetFactionInfoByID(self.factionID))

	-- prioritize 100% guild tabard if available
	if self.factionID == 1168 then
		if IsUsableItem(69210) then self.tabardID = 69210 else self.tabardID = 69209 end
	end

	ExpandAllFactionHeaders()

	for index = 1, GetNumFactions() do
		local factionIndexName = select(1, GetFactionInfo(index))
		if factionName == factionIndexName then
			if IsUsableItem(self.tabardID) then
				SetWatchedFactionIndex(index)
				EquipItemByName(self.tabardID)
				DEFAULT_CHAT_FRAME:AddMessage("|cffFFFF00Watching: |r"..factionName.."")
			else
				DEFAULT_CHAT_FRAME:AddMessage("|cffFF0000Can't find tabard in inventory|r")
			end
		end
	end
end

-- show/hide button
local tabardbtn = CreateFrame("Button", nil, UIParent)
tabardbtn:CreatePanel("Default", 16, 100, unpack(config.position))
tabardbtn:RegisterForClicks("AnyUp")
tabardbtn.text = tabardbtn:CreateFontString(nil, 'OVERLAY')
tabardbtn.text:SetFont(font, config.fontSize, config.fontFlag)
tabardbtn.text:SetText(">")
tabardbtn.text:SetPoint("CENTER", 1, 0)
tabardbtn:SetAlpha(0)

-- the actual frame
local tabard = CreateFrame("Frame", TukuiTabard, UIParent)
tabard:CreatePanel("Default", 1, 1, "LEFT", tabardbtn, "RIGHT", 1, -10)
tabard:SetSize((btnw) + (btns*2), (btnh*16) + (btns*17))
tabard:Hide()

-- title frame
local tabardtitle = CreateFrame("Frame", nil, tabard)
tabardtitle:CreatePanel("Default", btnw + btns + btns, btnh, "BOTTOM", tabard, "TOP", 0, 1)
tabardtitle.text = tabardtitle:CreateFontString(nil, 'OVERLAY')
tabardtitle.text:SetFont(font, config.fontSize, config.fontFlag)
tabardtitle.text:SetText("Faction Championing")
tabardtitle.text:SetPoint("CENTER", 1, 0)

-- script for showing/hiding the frames
tabardbtn:SetScript("OnEnter", function(self) self:SetAlpha(1) end)
tabardbtn:SetScript("OnLeave", function(self) if not tabard:IsShown() then self:SetAlpha(0) end end)
tabardbtn:SetScript("OnClick", function(self)
	if tabard:IsShown() then 
		self.text:SetText(">") 
		tabard:Hide() 
	else 
		self.text:SetText("<") 
		tabard:Show() 
	end
end)

-- parent frames
local hordeframe = CreateFrame("Frame", nil, tabard)
local allianceframe = CreateFrame("Frame", nil, tabard)
local wrathframe = CreateFrame("Frame", nil, tabard)
local cataframe = CreateFrame("Frame", nil, tabard)

-- Horde
local horde = {}
for i = 1, 6 do
	local btn = horde[i]

	btn = CreateFrame("Button", nil, hordeframe)

	local info = E.HordeFactions[i]
	btn.factionID, btn.tabardID = info[1], info[2]

	btn:CreatePanel("Default", btnw, btnh, "CENTER", tabard, "CENTER", 0, 0)

	btn:SetPoint("TOPLEFT", tabard, btns, -(btnh*(i-1))-(btns*i))

	btn.bar = CreateFrame("StatusBar", nil, btn)
	btn.bar:SetStatusBarTexture(C.media.normTex)
	btn.bar:SetPoint("TOPLEFT", 2, -2)
	btn.bar:SetPoint("BOTTOMRIGHT", -2, 2)

	btn.text = btn.bar:CreateFontString(nil, 'OVERLAY')
	btn.text:SetFont(font, config.fontSize, config.fontFlag)
	btn.text:SetSize(btnw, btnh)
	btn.text:SetPoint("CENTER", 1, 0)
	btn.text:SetJustifyH("CENTER")

	btn:RegisterEvent("PLAYER_LOGIN")
	btn:RegisterEvent("UPDATE_FACTION")
	btn:SetScript("OnEvent", E.FactionInfo)

	btn:RegisterForClicks("AnyUp")
	btn:SetScript("OnClick", E.FactionSwitch)
end

local alliance = {}
for i = 1, 6 do
	local btn = alliance[i]

	btn = CreateFrame("Button", nil, allianceframe)

	local info = E.AllianceFactions[i]
	btn.factionID, btn.tabardID = info[1], info[2]

	btn:CreatePanel("Default", btnw, btnh, "CENTER", tabard, "CENTER", 0, 0)

	btn:SetPoint("TOPLEFT", tabard, btns, -(btnh*(i-1))-(btns*i))

	btn.bar = CreateFrame("StatusBar", nil, btn)
	btn.bar:SetStatusBarTexture(C.media.normTex)
	btn.bar:SetPoint("TOPLEFT", 2, -2)
	btn.bar:SetPoint("BOTTOMRIGHT", -2, 2)

	btn.text = btn.bar:CreateFontString(nil, 'OVERLAY')
	btn.text:SetFont(font, config.fontSize, config.fontFlag)
	btn.text:SetSize(btnw, btnh)
	btn.text:SetPoint("CENTER", 1, 0)
	btn.text:SetJustifyH("CENTER")

	btn:RegisterEvent("PLAYER_LOGIN")
	btn:RegisterEvent("UPDATE_FACTION")
	btn:SetScript("OnEvent", E.FactionInfo)

	btn:RegisterForClicks("AnyUp")
	btn:SetScript("OnClick", E.FactionSwitch)
end

local wrath = {}
for i = 1, 4 do
	local btn = wrath[i]

	btn = CreateFrame("Button", nil, wrathframe)

	local info = E.WrathFactions[i]
	btn.factionID, btn.tabardID = info[1], info[2]

	btn:CreatePanel("Default", btnw, btnh, "CENTER", tabard, "CENTER", 0, 0)

	btn:SetPoint("TOPLEFT", tabard, btns, -(btnh*(i+5))-(btns*(i+6)))

	btn.bar = CreateFrame("StatusBar", nil, btn)
	btn.bar:SetStatusBarTexture(C.media.normTex)
	btn.bar:SetPoint("TOPLEFT", 2, -2)
	btn.bar:SetPoint("BOTTOMRIGHT", -2, 2)

	btn.text = btn.bar:CreateFontString(nil, 'OVERLAY')
	btn.text:SetFont(font, config.fontSize, config.fontFlag)
	btn.text:SetSize(btnw, btnh)
	btn.text:SetPoint("CENTER", 1, 0)
	btn.text:SetJustifyH("CENTER")

	btn:RegisterEvent("PLAYER_LOGIN")
	btn:RegisterEvent("UPDATE_FACTION")
	btn:SetScript("OnEvent", E.FactionInfo)

	btn:RegisterForClicks("AnyUp")
	btn:SetScript("OnClick", E.FactionSwitch)
end

local cata = {}
for i = 1, 6 do
	local btn = cata[i]

	btn = CreateFrame("Button", nil, cataframe)

	local info = E.CataFactions[i]
	if i == 5 and faction == "Alliance" then info = E.CataFactions[7] end
	btn.factionID, btn.tabardID = info[1], info[2]

	btn:CreatePanel("Default", btnw, btnh, "CENTER", tabard, "CENTER", 0, 0)

	btn:SetPoint("TOPLEFT", tabard, btns, -(btnh*(i+9))-(btns*(i+10)))

	btn.bar = CreateFrame("StatusBar", nil, btn)
	btn.bar:SetStatusBarTexture(C.media.normTex)
	btn.bar:SetPoint("TOPLEFT", 2, -2)
	btn.bar:SetPoint("BOTTOMRIGHT", -2, 2)

	btn.text = btn.bar:CreateFontString(nil, 'OVERLAY')
	btn.text:SetFont(font, config.fontSize, config.fontFlag)
	btn.text:SetSize(btnw, btnh)
	btn.text:SetPoint("CENTER", 1, 0)
	btn.text:SetJustifyH("CENTER")

	btn:RegisterEvent("PLAYER_LOGIN")
	btn:RegisterEvent("UPDATE_FACTION")
	btn:SetScript("OnEvent", E.FactionInfo)

	btn:RegisterForClicks("AnyUp")
	btn:SetScript("OnClick", E.FactionSwitch)
end

if faction == "Horde" then
	allianceframe:Hide()
else
	hordeframe:Hide()
end